﻿


DeclareModule Lizard
	
	Enumeration Event 1
		#Event_Print
		#Event_Message
		#Event_Error
	EndEnumeration
	
	Declare.i Initialize(LibraryPath.s="")             ; Initializes the kernel of Lizard. Define with LibraryPath the path of the Lizard.dll location.
	Declare.i Reset()                                  ; Resets the kernel of Lizard to the state after initialization. This function is faster than terminate and initialize the kernel again.
	Declare.i Terminate()                              ; Terminates the kernel of Lizard and releases all resources. 
	Declare.s Version()                                ; Give the version number and processor type of the kernel as a string.
	Declare.i InputExpression(String.s)                ; Parses a string and returns the address to the expression or #Null if an error occurs.
	Declare.i DuplicateExpression(*Expression)         ; Dublicates a valid expression and returns the address to new expression. The source expression keeps valid. 
	Declare.i EvaluateExpression(*Expression)          ; Evaluates a valid expression and returns a new expression. The old expression is not longer valid.
	Declare.i FreeExpression(*Expression)              ; Frees a valid expression to clean up the kernel.
	Declare.s OutputExpression(*Expression)            ; Gives a string representation of a valid expression. The expression keeps valid. 
	Declare.s Evaluate(Input.s)                        ; Short form of parsing a string, evaluating the expression, and getting the result as a string.
	Declare.s EvaluateFile(FileName.s)                 ; Parses and evaluates a file with a Lizard script and gives the output of this script as a string.
	Declare.i BindCallback(Event.i, *CallbackAddress)  ; Bind a callback procedure for one of the following events: #Event_Print, #Event_Message, #Event_Error
	
	Declare.i SetQuad(SymbolName.s, Integer.q)         ; Assigns a PB-Quad as the content of the specified symbol. The symbol content is then an Integer object.
	Declare.i SetDouble(SymbolName.s, Double.d)        ; Assigns a PB-Double as the content of the specified symbol. The symbol content is then a Real object.
	Declare.i SetString(SymbolName.s, String.s)        ; Assigns a PB-String as the content of the specified symbol. The symbol content is then a (text) String object.
	Declare.i SetMemory(SymbolName.s, *Memory, MemorySize.i=#PB_Default)
	                                                   ; Assigns a memory buffer as the content of the specified symbol. The symbol content is then a (binary) String object.
	
	Declare.q GetQuad(SymbolName.s)                    ;{ Returns a PB-Quad from the content of the specified symbol. 
	                                                   ;    The symbol must be exist and its content must be an Integer, Rational Or Real object, otherwise 0 is returned.
	                                                   ;}
	Declare.d GetDouble(SymbolName.s)                  ;{ Returns a PB-Double from the content of the specified symbol.
	                                                   ;    The symbol must be exist and its content must be an Integer, Rational Or Real object, otherwise 0.0 is returned.
	                                                   ;}
	Declare.s GetString(SymbolName.s)                  ;{ Returns a PB-String from the content of the specified symbol.
	                                                   ;    The symbol must be exist and its content must be String object, otherwise #Empty$ is returned.
	                                                   ;    The string is truncated if a NUL-character is contained. Use GetMemory instead.
	                                                   ;}
	Declare.i GetMemory(SymbolName.s, *AllocatedMemory=#Null, AllocatedSize.i=#PB_Ignore)
	                                                   ;{ Returns the address of an PB-allocated memory buffer from the content of the specified symbol.
	                                                   ;    The symbol must be exist and its content must be String object, otherwise #Null is returned.
	                                                   ;    It returns also #Null if the buffer has no length.
	                                                   ;    If *AllocatedMemory is given, the content will stored in this address and no additional memory is allocated.
	                                                   ;    Use AllocatedSize to define a max size of the *AllocatedMemory to prevent a write overflow.
	                                                   ;}
	
EndDeclareModule



Module Lizard
	
	EnableExplicit
	
	Enumeration Lizard_Type 1
		#Lizard_Type_Quad
		#Lizard_Type_Double
		#Lizard_Type_String
		#Lizard_Type_Memory
	EndEnumeration
	
	Global LizardLibrary.i
	
	Prototype.i Lizard_Initialize()
	Prototype.i Lizard_Reset()
	Prototype.i Lizard_Terminate()
	Prototype.q Lizard_Version()
	Prototype.i Lizard_InputExpression(String.s)
	Prototype.i Lizard_DuplicateExpression(*Expression)
	Prototype.i Lizard_EvaluateExpression(*Expression)
	Prototype.i Lizard_FreeExpression(*Expression)
	Prototype.i Lizard_OutputExpression(*Expression)
	Prototype.i Lizard_BindCallback(Event.i, *CallbackAddress)
	Prototype.i Lizard_SetValue(SymbolName.s, Type.i, *Buffer, BufferLength.i=0)
	Prototype.i Lizard_GetValue(SymbolName.s, Type.i, *Buffer.Integer, *BufferLength.Integer=#Null)
	
	
	Procedure.i Initialize(LibraryPath.s="")
		
		If LizardLibrary = #Null
			If LibraryPath <> "" And Left(LibraryPath, 1) <> #PS$ : LibraryPath + #PS$ : EndIf 
			CompilerSelect #PB_Compiler_OS
				CompilerCase #PB_OS_Windows
					CompilerSelect #PB_Compiler_Processor
						CompilerCase #PB_Processor_x64
							LizardLibrary = OpenLibrary(#PB_Any, LibraryPath+"Lizard.dll")
						CompilerCase #PB_Processor_x86
							LizardLibrary = OpenLibrary(#PB_Any, LibraryPath+"Lizard_x86.dll")
					CompilerEndSelect
				CompilerCase #PB_OS_Linux
					CompilerSelect #PB_Compiler_Processor
						CompilerCase #PB_Processor_x64
							LizardLibrary = OpenLibrary(#PB_Any, LibraryPath+"Lizard.so")
						CompilerCase #PB_Processor_x86
							CompilerError "Lizard is not compatible with Linux on a x86 processor. Try it on a x64 processor."
					CompilerEndSelect
				CompilerCase #PB_OS_MacOS
					CompilerError "Lizard is not compatible with MacOS."
			CompilerEndSelect
		EndIf
		
		If LizardLibrary
			Global Lizard_Initialize.Lizard_Initialize                  = GetFunction(LizardLibrary, "Lizard_Initialize")
			Global Lizard_Reset.Lizard_Reset                            = GetFunction(LizardLibrary, "Lizard_Reset")
			Global Lizard_Terminate.Lizard_Terminate                    = GetFunction(LizardLibrary, "Lizard_Terminate")
			Global Lizard_Version.Lizard_Version                        = GetFunction(LizardLibrary, "Lizard_Version")
			Global Lizard_InputExpression.Lizard_InputExpression        = GetFunction(LizardLibrary, "Lizard_InputExpression")
			Global Lizard_DuplicateExpression.Lizard_EvaluateExpression = GetFunction(LizardLibrary, "Lizard_DuplicateExpression")
			Global Lizard_EvaluateExpression.Lizard_EvaluateExpression  = GetFunction(LizardLibrary, "Lizard_EvaluateExpression")
			Global Lizard_FreeExpression.Lizard_FreeExpression          = GetFunction(LizardLibrary, "Lizard_FreeExpression")
			Global Lizard_OutputExpression.Lizard_OutputExpression      = GetFunction(LizardLibrary, "Lizard_OutputExpression")
			Global Lizard_BindCallback.Lizard_BindCallback              = GetFunction(LizardLibrary, "Lizard_BindCallback")
			Global Lizard_SetValue.Lizard_SetValue                      = GetFunction(LizardLibrary, "Lizard_SetValue")
			Global Lizard_GetValue.Lizard_GetValue                      = GetFunction(LizardLibrary, "Lizard_GetValue")
		EndIf
		
		If Lizard_Initialize And Lizard_Reset And Lizard_Terminate And Lizard_Version And Lizard_InputExpression And Lizard_DuplicateExpression And
		   Lizard_EvaluateExpression And Lizard_OutputExpression And Lizard_BindCallback And Lizard_SetValue And Lizard_GetValue
			ProcedureReturn Lizard_Initialize()
		EndIf
		
	EndProcedure
	
	Procedure.i Reset()
		
		ProcedureReturn Lizard_Reset()
		
	EndProcedure
	
	Procedure.i Terminate()
		
		Lizard_Terminate()
		If LizardLibrary
			Lizard_Version             = #Null
			Lizard_Initialize          = #Null
			Lizard_Terminate           = #Null
			Lizard_InputExpression     = #Null
			Lizard_DuplicateExpression = #Null
			Lizard_EvaluateExpression  = #Null
			Lizard_FreeExpression      = #Null
			Lizard_OutputExpression    = #Null
			Lizard_BindCallback        = #Null
			Lizard_SetValue            = #Null
			Lizard_GetValue            = #Null
			CloseLibrary(LizardLibrary)
			LizardLibrary = #Null
		EndIf
		
	EndProcedure
	
	Procedure.s Version()
		
		Protected Version.q = Lizard_Version()
		
		CompilerSelect #PB_Compiler_OS
			CompilerCase #PB_OS_Windows
				CompilerSelect #PB_Compiler_Processor
					CompilerCase #PB_Processor_x64
						ProcedureReturn Str(Version>>48 & $FF) + "." + Str(Version>>32 & $FF) + "." + Str(Version>>16 & $FF) + "-" + RSet(Str(Version & $FF), 4, "0") + " (64-bit, Windows)"
					CompilerCase #PB_Processor_x86
						ProcedureReturn Str(Version>>48 & $FF) + "." + Str(Version>>32 & $FF) + "." + Str(Version>>16 & $FF) + "-" + RSet(Str(Version & $FF), 4, "0") + " (32-bit, Windows)"
				CompilerEndSelect
			CompilerCase #PB_OS_Linux
				ProcedureReturn Str(Version>>48 & $FF) + "." + Str(Version>>32 & $FF) + "." + Str(Version>>16 & $FF) + "-" + RSet(Str(Version & $FF), 4, "0") + " (64-bit, Linux)"
		CompilerEndSelect
		
	EndProcedure
	
	Procedure.i InputExpression(String.s)
		
		ProcedureReturn Lizard_InputExpression(String)
		
	EndProcedure
	
	Procedure.i DuplicateExpression(*Expression)
		
		ProcedureReturn Lizard_DuplicateExpression(*Expression)
		
	EndProcedure
	
	Procedure.i EvaluateExpression(*Expression)
		
		ProcedureReturn Lizard_EvaluateExpression(*Expression)
		
	EndProcedure
			
	Procedure.i FreeExpression(*Expression)
		
		ProcedureReturn Lizard_FreeExpression(*Expression)
		
	EndProcedure
	
	Procedure.s OutputExpression(*Expression)
		
		ProcedureReturn PeekS(Lizard_OutputExpression(*Expression), -1, #PB_Unicode)
		
	EndProcedure
	
	Procedure.s Evaluate(Input.s)
		
		Protected *Expression, Result.s
		
		*Expression = InputExpression(Input)
		If *Expression
			*Expression = EvaluateExpression(*Expression)
			If *Expression
				Result = OutputExpression(*Expression)
				FreeExpression(*Expression)
				ProcedureReturn Result
			EndIf
		EndIf
		
	EndProcedure
	
	Procedure.s EvaluateFile(FileName.s)
		
		Protected File.i = ReadFile(#PB_Any, FileName, #PB_File_SharedRead)
		
		If File
			Select ReadStringFormat(File)
				Case #PB_Ascii
					ProcedureReturn Evaluate(ReadString(File, #PB_Ascii|#PB_File_IgnoreEOL))
				Case #PB_Unicode
					ProcedureReturn Evaluate(ReadString(File, #PB_Unicode|#PB_File_IgnoreEOL))
				Case #PB_UTF8
					ProcedureReturn Evaluate(ReadString(File, #PB_UTF8|#PB_File_IgnoreEOL))
			EndSelect
			CloseFile(File)
		EndIf
		
		ProcedureReturn #Empty$
		
	EndProcedure
	
	Procedure.i BindCallback(Event.i, *CallbackAddress)
		
		ProcedureReturn Lizard_BindCallback(Event, *CallbackAddress)
		
	EndProcedure
	
	Procedure.i SetQuad(SymbolName.s, Quad.q)
		
		ProcedureReturn Lizard_SetValue(SymbolName, #Lizard_Type_Quad, @Quad)
		
	EndProcedure
	
	Procedure.i SetDouble(SymbolName.s, Double.d)
		
		ProcedureReturn Lizard_SetValue(SymbolName, #Lizard_Type_Double, @Double)
		
	EndProcedure
	
	Procedure.i SetString(SymbolName.s, String.s)
		
		Protected *Buffer = UTF8(String)
		Protected Result.i = Lizard_SetValue(SymbolName, #Lizard_Type_String, *Buffer, MemorySize(*Buffer)-1)
		
		FreeMemory(*Buffer)
		
		ProcedureReturn Result
		
	EndProcedure
	
	Procedure.i SetMemory(SymbolName.s, *Memory, MemorySize.i=#PB_Default)
		
		If MemorySize = #PB_Default And *Memory
			MemorySize = MemorySize(*Memory)
		EndIf
		
		ProcedureReturn Lizard_SetValue(SymbolName, #Lizard_Type_Memory, *Memory, MemorySize)
		
	EndProcedure
	
	Procedure.q GetQuad(SymbolName.s)
		
		Protected Quad.q = 0
		
		Lizard_GetValue(SymbolName, #Lizard_Type_Quad, @Quad)
		
		ProcedureReturn Quad
		
	EndProcedure
	
	Procedure.d GetDouble(SymbolName.s)
		
		Protected Double.d = 0.0
		
		Lizard_GetValue(SymbolName, #Lizard_Type_Double, @Double)
		
		ProcedureReturn Double
		
	EndProcedure
	
	Procedure.s GetString(SymbolName.s)
		
		Protected *String, Length.i
		
		Select Lizard_GetValue(SymbolName, #Lizard_Type_String, @*String, @Length)
			Case #Lizard_Type_String
				ProcedureReturn PeekS(*String, Length, #PB_UTF8|#PB_ByteLength)
			Case #Lizard_Type_Memory
				ProcedureReturn PeekS(*String, Length, #PB_Ascii)
		EndSelect
		
		ProcedureReturn #Empty$
		
	EndProcedure
	
	Procedure.i GetMemory(SymbolName.s, *AllocatedMemory=#Null, AllocatedSize.i=#PB_Ignore)
		
		Protected *Memory, Length.i
		
		If Lizard_GetValue(SymbolName, #Lizard_Type_Memory, @*Memory, @Length)
			If Length
				If *AllocatedMemory = #Null
					*AllocatedMemory = AllocateMemory(Length, #PB_Memory_NoClear)
				ElseIf AllocatedSize <> #PB_Ignore And AllocatedSize < Length
					Length = AllocatedSize
				EndIf
				CopyMemory(*Memory, *AllocatedMemory, Length)
				ProcedureReturn *AllocatedMemory
			EndIf
		EndIf
		
		ProcedureReturn #Null
		
	EndProcedure
	
	
EndModule



; IDE Options = PureBasic 6.00 LTS (Windows - x64)
; Folding = hAAA9
; EnableXP
; DPIAware
; EnableCompileCount = 9
; EnableBuildCount = 0